"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var validator_1 = require("../../validator");
var factory_1 = require("../factory");
function createController(option, controllerInfo, parameters) {
    var validator = new validator_1.ValidatorImpl(option.metaDataStorage, option.validators);
    validator.setValue(parameters, controllerInfo.classMetaData, controllerInfo.methodMetaData.name);
    var controller = factory_1.ControllerFactory.resolve(controllerInfo, option.dependencyResolver);
    controller.validator = validator;
    return controller;
}
var ControllerInvocation = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerInvocation, _super);
    function ControllerInvocation() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControllerInvocation.prototype.proceed = function () {
        var controller = createController(this.facade, this.controllerInfo, this.parameters);
        controller.context = this.context;
        var method = controller[this.controllerInfo.methodMetaData.name];
        var result;
        if (this.facade.autoValidation && this.controllerInfo.methodMetaData.parameters.length > 0 && !controller.validator.isValid())
            result = new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json");
        else
            result = method.apply(controller, this.parameters);
        return this.createResult(result);
    };
    ControllerInvocation.prototype.createResult = function (result) {
        var _this = this;
        return Promise.resolve(result)
            .then(function (awaitedResult) {
            if (awaitedResult instanceof Core.ActionResult)
                return awaitedResult;
            if (_this.controllerInfo.classMetaData.baseClass == "ApiController") {
                return new Core.ActionResult(awaitedResult, 200, "application/json");
            }
            return new Core.ActionResult(awaitedResult, 200, "text/html");
        });
    };
    return ControllerInvocation;
}(Core.Invocation));
exports.ControllerInvocation = ControllerInvocation;
