"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Core = require("kamboja-core");
var invocation_1 = require("./invocation");
var factory_1 = require("./factory");
var framework_1 = require("../framework");
var index_1 = require("../binder/index");
var RequestHandler = /** @class */ (function () {
    function RequestHandler(facade, controllerInfo) {
        this.facade = facade;
        this.controllerInfo = controllerInfo;
        //let middlewares = <Core.Middleware[]>(facade.middlewares || []).slice().reverse()
        var middlewares = [];
        this.tail = this.head = new invocation_1.AttachableInvocation(middlewares, controllerInfo, facade);
        if (facade.middlewares) {
            var i = facade.middlewares.length;
            while (i--) {
                var mdw = facade.middlewares[i];
                middlewares.push(mdw);
                this.tail = new invocation_1.MiddlewareInvocation(mdw, this.tail, middlewares, facade, controllerInfo);
            }
        }
        if (controllerInfo) {
            var controller = factory_1.ControllerFactory.resolve(controllerInfo, facade.dependencyResolver);
            var ctrDecorators = framework_1.MiddlewareDecorator.getMiddlewares(controller);
            if (ctrDecorators) {
                for (var _i = 0, ctrDecorators_1 = ctrDecorators; _i < ctrDecorators_1.length; _i++) {
                    var middleware = ctrDecorators_1[_i];
                    var mdw = factory_1.MiddlewareFactory.resolve(middleware, facade.dependencyResolver);
                    middlewares.push(mdw);
                    this.tail = new invocation_1.MiddlewareInvocation(mdw, this.tail, middlewares, facade, controllerInfo);
                }
            }
            var mtdMiddleware = framework_1.MiddlewareDecorator.getMiddlewares(controller, controllerInfo.methodMetaData.name);
            if (mtdMiddleware) {
                for (var _a = 0, mtdMiddleware_1 = mtdMiddleware; _a < mtdMiddleware_1.length; _a++) {
                    var middleware = mtdMiddleware_1[_a];
                    var mdw = factory_1.MiddlewareFactory.resolve(middleware, facade.dependencyResolver);
                    middlewares.push(mdw);
                    this.tail = new invocation_1.MiddlewareInvocation(mdw, this.tail, middlewares, facade, controllerInfo);
                }
            }
        }
    }
    RequestHandler.prototype.execute = function (context, response, invocation) {
        var _this = this;
        return new Promise(function (resolve) { return resolve(_this.prepare(context, invocation)); })
            .then(function (result) { return _this.tail.proceed(); })
            .then(function (result) {
            if (context.contextType == "Handshake" && result.engine != "General")
                throw new Error("ActionResult Error, only return value of type 'redirect' and 'emit' are allowed in real time action");
            return result.execute(context, response, invocation.controllerInfo);
        })
            .catch(function (e) {
            response.send(new Core.ActionResult(e.message, e.status || 500));
        });
    };
    RequestHandler.prototype.prepare = function (context, invocation) {
        this.head.attach(invocation);
        if (context.contextType == "HttpRequest" && this.controllerInfo && this.controllerInfo.methodMetaData.parameters.length > 0) {
            var binder = new index_1.ParameterBinder(this.controllerInfo, this.facade.pathResolver);
            this.tail.parameters = binder.getParameters(context);
        }
        else if (context.contextType == "Handshake") {
            this.tail.parameters = [context.getPacket()];
        }
        this.tail.context = context;
    };
    return RequestHandler;
}());
exports.RequestHandler = RequestHandler;
